@@ -19,6 +19,12 @@
 #include "envtoconfitems.h"
 #include "ccache.h"
 
+enum handle_conf_result {
+	HANDLE_CONF_OK,
+	HANDLE_CONF_UNKNOWN,
+	HANDLE_CONF_FAIL
+};
+
 static const struct conf_item *
 find_conf(const char *name)
 {
@@ -31,15 +37,14 @@ find_env_to_conf(const char *name)
 	return envtoconfitems_get(name, strlen(name));
 }
 
-static bool
+static enum handle_conf_result
 handle_conf_setting(struct conf *conf, const char *key, const char *value,
                     char **errmsg, bool from_env_variable, bool negate_boolean,
                     const char *origin)
 {
 	const struct conf_item *item = find_conf(key);
 	if (!item) {
-		*errmsg = format("unknown configuration option \"%s\"", key);
-		return false;
+		return HANDLE_CONF_UNKNOWN;
 	}
 
 	if (from_env_variable && item->parser == confitem_parse_bool) {
@@ -61,21 +66,21 @@ handle_conf_setting(struct conf *conf, const char *key, const char *value,
 	}
 
 	if (!item->parser(value, (char *)conf + item->offset, errmsg)) {
-		return false;
+		return HANDLE_CONF_FAIL;
 	}
 	if (item->verifier && !item->verifier((char *)conf + item->offset, errmsg)) {
-		return false;
+		return HANDLE_CONF_FAIL;
 	}
 
 out:
 	conf->item_origins[item->number] = origin;
-	return true;
+	return HANDLE_CONF_OK;
 }
 
 static bool
 parse_line(const char *line, char **key, char **value, char **errmsg)
 {
-#define SKIP_WS(x) while (isspace(*x)) { ++x; }
+#define SKIP_WS(x) do { while (isspace(*x)) { ++x; } } while (false)
 
 	*key = NULL;
 	*value = NULL;
@@ -155,7 +160,6 @@ conf_create(void)
 	conf->stats = true;
 	conf->temporary_dir = x_strdup("");
 	conf->umask = UINT_MAX; // Default: don't set umask.
-	conf->unify = false;
 	conf->item_origins = x_malloc(confitems_count() * sizeof(char *));
 	for (size_t i = 0; i < confitems_count(); ++i) {
 		conf->item_origins[i] = "default";
@@ -187,7 +191,7 @@ conf_free(struct conf *conf)
 
 // Note: The path pointer is stored in conf, so path must outlive conf.
 //
-// On failure, if an I/O error occured errno is set appropriately, otherwise
+// On failure, if an I/O error occurred errno is set appropriately, otherwise
 // errno is set to zero indicating that config itself was invalid.
 bool
 conf_read(struct conf *conf, const char *path, char **errmsg)
@@ -210,9 +214,12 @@ conf_read(struct conf *conf, const char *path, char **errmsg)
 		char *key;
 		char *value;
 		char *errmsg2;
+		enum handle_conf_result hcr = HANDLE_CONF_OK;
 		bool ok = parse_line(buf, &key, &value, &errmsg2);
 		if (ok && key) { // key == NULL if comment or blank line.
-			ok = handle_conf_setting(conf, key, value, &errmsg2, false, false, path);
+			hcr =
+				handle_conf_setting(conf, key, value, &errmsg2, false, false, path);
+			ok = hcr != HANDLE_CONF_FAIL; // unknown is OK
 		}
 		free(key);
 		free(value);
@@ -265,11 +272,11 @@ conf_update_from_environment(struct conf *conf, char **errmsg)
 			continue;
 		}
 
-		char *errmsg2;
-		bool ok = handle_conf_setting(
+		char *errmsg2 = NULL;
+		enum handle_conf_result hcr = handle_conf_setting(
 			conf, env_to_conf_item->conf_name, q, &errmsg2, true, negate,
 			"environment");
-		if (!ok) {
+		if (hcr != HANDLE_CONF_OK) {
 			*errmsg = format("%s: %s", key, errmsg2);
 			free(errmsg2);
 			free(key);
@@ -292,6 +299,12 @@ conf_set_value_in_file(const char *path, const char *key, const char *value,
 		return false;
 	}
 
+	char dummy[8] = {0}; // The maximum entry size in struct conf.
+	if (!item->parser(value, (void *)dummy, errmsg)
+	    || (item->verifier && !item->verifier(value, errmsg))) {
+		return false;
+	}
+
 	FILE *infile = fopen(path, "r");
 	if (!infile) {
 		*errmsg = format("%s: %s", path, strerror(errno));
@@ -415,6 +428,5 @@ conf_print_items(struct conf *conf,
 	ok &= print_item(conf, "stats", printer, context);
 	ok &= print_item(conf, "temporary_dir", printer, context);
 	ok &= print_item(conf, "umask", printer, context);
-	ok &= print_item(conf, "unify", printer, context);
 	return ok;
 }
