@@ -1,4 +1,4 @@
-// Copyright (C) 2009-2018 Joel Rosdahl
+// Copyright (C) 2009-2019 Joel Rosdahl
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the Free
@@ -67,7 +67,7 @@ static const uint32_t MAX_MANIFEST_ENTRIES = 100;
 static const uint32_t MAX_MANIFEST_FILE_INFO_ENTRIES = 10000;
 
 #define ccache_static_assert(e) \
-  do { enum { ccache_static_assert__ = 1/(e) }; } while (false)
+	do { enum { ccache_static_assert__ = 1/(e) }; } while (false)
 
 struct file_info {
 	// Index to n_files.
@@ -155,7 +155,7 @@ free_manifest(struct manifest *mf)
 }
 
 #define READ_BYTE(var) \
-  do { \
+	do { \
 		int ch_ = gzgetc(f); \
 		if (ch_ == EOF) { \
 			goto error; \
@@ -164,7 +164,7 @@ free_manifest(struct manifest *mf)
 	} while (false)
 
 #define READ_INT(size, var) \
-  do { \
+	do { \
 		uint64_t u_ = 0; \
 		for (size_t i_ = 0; i_ < (size); i_++) { \
 			int ch_ = gzgetc(f); \
@@ -178,7 +178,7 @@ free_manifest(struct manifest *mf)
 	} while (false)
 
 #define READ_STR(var) \
-  do { \
+	do { \
 		char buf_[1024]; \
 		size_t i_; \
 		for (i_ = 0; i_ < sizeof(buf_); i_++) { \
@@ -198,7 +198,7 @@ free_manifest(struct manifest *mf)
 	} while (false)
 
 #define READ_BYTES(n, var) \
-  do { \
+	do { \
 		for (size_t i_ = 0; i_ < (n); i_++) { \
 			int ch_ = gzgetc(f); \
 			if (ch_ == EOF) { \
@@ -224,27 +224,32 @@ create_empty_manifest(void)
 }
 
 static struct manifest *
-read_manifest(gzFile f)
+read_manifest(gzFile f, char **errmsg)
 {
+	*errmsg = NULL;
 	struct manifest *mf = create_empty_manifest();
 
 	uint32_t magic;
 	READ_INT(4, magic);
 	if (magic != MAGIC) {
-		cc_log("Manifest file has bad magic number %u", magic);
+		*errmsg = format("Manifest file has bad magic number %u", magic);
 		goto error;
 	}
 
 	READ_BYTE(mf->version);
 	if (mf->version != MANIFEST_VERSION) {
-		cc_log("Manifest file has unknown version %u", mf->version);
+		*errmsg = format(
+			"Unknown manifest version (actual %u, expected %u)",
+			mf->version,
+			MANIFEST_VERSION);
 		goto error;
 	}
 
 	READ_BYTE(mf->hash_size);
 	if (mf->hash_size != 16) {
 		// Temporary measure until we support different hash algorithms.
-		cc_log("Manifest file has unsupported hash size %u", mf->hash_size);
+		*errmsg =
+			format("Manifest file has unsupported hash size %u", mf->hash_size);
 		goto error;
 	}
 
@@ -271,8 +276,8 @@ read_manifest(gzFile f)
 	for (uint32_t i = 0; i < mf->n_objects; i++) {
 		READ_INT(4, mf->objects[i].n_file_info_indexes);
 		mf->objects[i].file_info_indexes =
-		  x_calloc(mf->objects[i].n_file_info_indexes,
-		           sizeof(*mf->objects[i].file_info_indexes));
+			x_calloc(mf->objects[i].n_file_info_indexes,
+			         sizeof(*mf->objects[i].file_info_indexes));
 		for (uint32_t j = 0; j < mf->objects[i].n_file_info_indexes; j++) {
 			READ_INT(4, mf->objects[i].file_info_indexes[j]);
 		}
@@ -283,13 +288,15 @@ read_manifest(gzFile f)
 	return mf;
 
 error:
-	cc_log("Corrupt manifest file");
+	if (!*errmsg) {
+		*errmsg = x_strdup("Corrupt manifest file");
+	}
 	free_manifest(mf);
 	return NULL;
 }
 
 #define WRITE_INT(size, var) \
-  do { \
+	do { \
 		uint64_t u_ = (var); \
 		uint8_t ch_; \
 		size_t i_; \
@@ -302,14 +309,14 @@ error:
 	} while (false)
 
 #define WRITE_STR(var) \
-  do { \
+	do { \
 		if (gzputs(f, var) == EOF || gzputc(f, '\0') == EOF) { \
 			goto error; \
 		} \
 	} while (false)
 
 #define WRITE_BYTES(n, var) \
-  do { \
+	do { \
 		size_t i_; \
 		for (i_ = 0; i_ < (n); i_++) { \
 			if (gzputc(f, (var)[i_]) == EOF) { \
@@ -383,7 +390,8 @@ verify_object(struct conf *conf, struct manifest *mf, struct object *obj,
 
 		// Clang stores the mtime of the included files in the precompiled header,
 		// and will error out if that header is later used without rebuilding.
-		if (guessed_compiler == GUESSED_CLANG
+		if ((guessed_compiler == GUESSED_CLANG
+		     || guessed_compiler == GUESSED_UNKNOWN)
 		    && output_is_precompiled_header
 		    && fi->mtime != st->mtime) {
 			cc_log("Precompiled header includes %s, which has a new mtime", path);
@@ -391,30 +399,41 @@ verify_object(struct conf *conf, struct manifest *mf, struct object *obj,
 		}
 
 		if (conf->sloppiness & SLOPPY_FILE_STAT_MATCHES) {
-			if (fi->mtime == st->mtime && fi->ctime == st->ctime) {
-				cc_log("mtime/ctime hit for %s", path);
-				continue;
+			if (!(conf->sloppiness & SLOPPY_FILE_STAT_MATCHES_CTIME)) {
+				if (fi->mtime == st->mtime && fi->ctime == st->ctime) {
+					cc_log("mtime/ctime hit for %s", path);
+					continue;
+				} else {
+					cc_log("mtime/ctime miss for %s", path);
+				}
 			} else {
-				cc_log("mtime/ctime miss for %s", path);
+				if (fi->mtime == st->mtime) {
+					cc_log("mtime hit for %s", path);
+					continue;
+				} else {
+					cc_log("mtime miss for %s", path);
+				}
 			}
 		}
 
 		struct file_hash *actual = hashtable_search(hashed_files, path);
 		if (!actual) {
-			struct mdfour hash;
-			hash_start(&hash);
-			int result = hash_source_code_file(conf, &hash, path);
+			struct hash *hash = hash_init();
+			int result = hash_source_code_file(conf, hash, path);
 			if (result & HASH_SOURCE_CODE_ERROR) {
 				cc_log("Failed hashing %s", path);
+				hash_free(hash);
 				return 0;
 			}
 			if (result & HASH_SOURCE_CODE_FOUND_TIME) {
+				hash_free(hash);
 				return 0;
 			}
 			actual = x_malloc(sizeof(*actual));
-			hash_result_as_bytes(&hash, actual->hash);
-			actual->size = hash.totalN;
+			hash_result_as_bytes(hash, actual->hash);
+			actual->size = hash_input_size(hash);
 			hashtable_insert(hashed_files, x_strdup(path), actual);
+			hash_free(hash);
 		}
 		if (memcmp(fi->hash, actual->hash, mf->hash_size) != 0
 		    || fi->size != actual->size) {
@@ -429,7 +448,7 @@ static struct hashtable *
 create_string_index_map(char **strings, uint32_t len)
 {
 	struct hashtable *h =
-	  create_hashtable(1000, hash_from_string, strings_equal);
+		create_hashtable(1000, hash_from_string, strings_equal);
 	for (uint32_t i = 0; i < len; i++) {
 		uint32_t *index = x_malloc(sizeof(*index));
 		*index = i;
@@ -442,7 +461,7 @@ static struct hashtable *
 create_file_info_index_map(struct file_info *infos, uint32_t len)
 {
 	struct hashtable *h =
-	  create_hashtable(1000, hash_from_file_info, file_infos_equal);
+		create_hashtable(1000, hash_from_file_info, file_infos_equal);
 	for (uint32_t i = 0; i < len; i++) {
 		struct file_info *fi = x_malloc(sizeof(*fi));
 		*fi = infos[i];
@@ -474,7 +493,8 @@ get_file_hash_index(struct manifest *mf,
                     char *path,
                     struct file_hash *file_hash,
                     struct hashtable *mf_files,
-                    struct hashtable *mf_file_infos)
+                    struct hashtable *mf_file_infos,
+                    bool save_timestamp)
 {
 	struct file_info fi;
 	fi.index = get_include_file_index(mf, path, mf_files);
@@ -487,9 +507,13 @@ get_file_hash_index(struct manifest *mf,
 	//
 	// st->ctime may be 0, so we have to check time_of_compilation against
 	// MAX(mtime, ctime).
+	//
+	// ccache only reads mtime/ctime if file_stat_match sloppiness is enabled, so
+	// mtimes/ctimes are stored as a dummy value (-1) if not enabled. This reduces
+	// the number of file_info entries for the common case.
 
 	struct stat file_stat;
-	if (stat(path, &file_stat) != -1
+	if (save_timestamp && stat(path, &file_stat) != -1
 	    && time_of_compilation > MAX(file_stat.st_mtime, file_stat.st_ctime)) {
 		fi.mtime = file_stat.st_mtime;
 		fi.ctime = file_stat.st_ctime;
@@ -512,7 +536,8 @@ get_file_hash_index(struct manifest *mf,
 
 static void
 add_file_info_indexes(uint32_t *indexes, uint32_t size,
-                      struct manifest *mf, struct hashtable *included_files)
+                      struct manifest *mf, struct hashtable *included_files,
+                      bool save_timestamp)
 {
 	if (size == 0) {
 		return;
@@ -520,17 +545,17 @@ add_file_info_indexes(uint32_t *indexes, uint32_t size,
 
 	// path --> index
 	struct hashtable *mf_files =
-	  create_string_index_map(mf->files, mf->n_files);
+		create_string_index_map(mf->files, mf->n_files);
 	// struct file_info --> index
 	struct hashtable *mf_file_infos =
-	  create_file_info_index_map(mf->file_infos, mf->n_file_infos);
+		create_file_info_index_map(mf->file_infos, mf->n_file_infos);
 	struct hashtable_itr *iter = hashtable_iterator(included_files);
 	uint32_t i = 0;
 	do {
 		char *path = hashtable_iterator_key(iter);
 		struct file_hash *file_hash = hashtable_iterator_value(iter);
 		indexes[i] = get_file_hash_index(mf, path, file_hash, mf_files,
-		                                 mf_file_infos);
+		                                 mf_file_infos, save_timestamp);
 		i++;
 	} while (hashtable_iterator_advance(iter));
 	assert(i == size);
@@ -542,7 +567,8 @@ add_file_info_indexes(uint32_t *indexes, uint32_t size,
 static void
 add_object_entry(struct manifest *mf,
                  struct file_hash *object_hash,
-                 struct hashtable *included_files)
+                 struct hashtable *included_files,
+                 bool save_timestamp)
 {
 	uint32_t n_objs = mf->n_objects;
 	mf->objects = x_realloc(mf->objects, (n_objs + 1) * sizeof(*mf->objects));
@@ -552,7 +578,8 @@ add_object_entry(struct manifest *mf,
 	uint32_t n_fii = hashtable_count(included_files);
 	obj->n_file_info_indexes = n_fii;
 	obj->file_info_indexes = x_malloc(n_fii * sizeof(*obj->file_info_indexes));
-	add_file_info_indexes(obj->file_info_indexes, n_fii, mf, included_files);
+	add_file_info_indexes(obj->file_info_indexes, n_fii, mf, included_files,
+	                      save_timestamp);
 	memcpy(obj->hash.hash, object_hash->hash, mf->hash_size);
 	obj->hash.size = object_hash->size;
 }
@@ -580,9 +607,11 @@ manifest_get(struct conf *conf, const char *manifest_path)
 		cc_log("Failed to gzdopen manifest file");
 		goto out;
 	}
-	mf = read_manifest(f);
+
+	char *errmsg;
+	mf = read_manifest(f, &errmsg);
 	if (!mf) {
-		cc_log("Error reading manifest file");
+		cc_log("%s", errmsg);
 		goto out;
 	}
 
@@ -619,7 +648,7 @@ out:
 // Returns true on success, otherwise false.
 bool
 manifest_put(const char *manifest_path, struct file_hash *object_hash,
-             struct hashtable *included_files)
+             struct hashtable *included_files, bool save_timestamp)
 {
 	int ret = 0;
 	gzFile f2 = NULL;
@@ -641,9 +670,12 @@ manifest_put(const char *manifest_path, struct file_hash *object_hash,
 			close(fd1);
 			goto out;
 		}
-		mf = read_manifest(f1);
+		char *errmsg;
+		mf = read_manifest(f1, &errmsg);
 		gzclose(f1);
 		if (!mf) {
+			cc_log("%s", errmsg);
+			free(errmsg);
 			cc_log("Failed to read manifest file; deleting it");
 			x_unlink(manifest_path);
 			mf = create_empty_manifest();
@@ -683,7 +715,7 @@ manifest_put(const char *manifest_path, struct file_hash *object_hash,
 		goto out;
 	}
 
-	add_object_entry(mf, object_hash, included_files);
+	add_object_entry(mf, object_hash, included_files, save_timestamp);
 	if (write_manifest(f2, mf)) {
 		gzclose(f2);
 		f2 = NULL;
@@ -729,9 +761,11 @@ manifest_dump(const char *manifest_path, FILE *stream)
 		close(fd);
 		goto out;
 	}
-	mf = read_manifest(f);
+	char *errmsg;
+	mf = read_manifest(f, &errmsg);
 	if (!mf) {
-		fprintf(stderr, "Error reading manifest file\n");
+		fprintf(stderr, "%s\n", errmsg);
+		free(errmsg);
 		goto out;
 	}
 
