@@ -24,8 +24,8 @@ TEST_SUITE(compopt)
 
 TEST(option_table_should_be_sorted)
 {
-	bool compopt_verify_sortedness(void);
-	CHECK(compopt_verify_sortedness());
+	bool compopt_verify_sortedness_and_flags(void);
+	CHECK(compopt_verify_sortedness_and_flags());
 }
 
 TEST(dash_I_affects_cpp)
@@ -125,4 +125,14 @@ TEST(dash_iframework_prefix_affects_cpp)
 	CHECK(compopt_prefix_affects_cpp("-iframework"));
 }
 
+TEST(dash_analyze_too_hard)
+{
+	CHECK(compopt_too_hard("-analyze"));
+}
+
+TEST(dash_dash_analyze_too_hard)
+{
+	CHECK(compopt_too_hard("--analyze"));
+}
+
 TEST_SUITE_END
